@@ -18,12 +18,15 @@
 
 package org.apache.jute;
 
+import org.apache.yetus.audience.InterfaceAudience;
+
 import java.io.IOException;
 
 /**
  * Interface that is implemented by generated classes.
  * 
  */
+@InterfaceAudience.Public
 public interface Record {
     public void serialize(OutputArchive archive, String tag)
         throws IOException;
