@@ -35,6 +35,7 @@ import java.util.concurrent.LinkedBlockingQueue;
 import org.apache.jute.Record;
 import org.apache.jute.BinaryOutputArchive;
 
+import org.apache.zookeeper.common.Time;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.apache.zookeeper.CreateMode;
@@ -318,7 +319,7 @@ public class PrepRequestProcessor extends ZooKeeperCriticalThread implements
         throws KeeperException, IOException, RequestProcessorException
     {
         request.hdr = new TxnHeader(request.sessionId, request.cxid, zxid,
-                                    zks.getTime(), type);
+                                    Time.currentWallTime(), type);
 
         switch (type) {
             case OpCode.create:                
@@ -504,6 +505,8 @@ public class PrepRequestProcessor extends ZooKeeperCriticalThread implements
                 version = currentVersion + 1;
                 request.txn = new CheckVersionTxn(path, version);
                 break;
+            default:
+                LOG.error("Invalid OpCode: {} received by PrepRequestProcessor", type);
         }
     }
 
@@ -557,9 +560,9 @@ public class PrepRequestProcessor extends ZooKeeperCriticalThread implements
                 try {
                     ByteBufferInputStream.byteBuffer2Record(request.request, multiRequest);
                 } catch(IOException e) {
-                   request.hdr =  new TxnHeader(request.sessionId, request.cxid, zks.getNextZxid(),
-                            zks.getTime(), OpCode.multi);
-                   throw e;
+                    request.hdr =  new TxnHeader(request.sessionId, request.cxid, zks.getNextZxid(),
+                            Time.currentWallTime(), OpCode.multi);
+                    throw e;
                 }
                 List<Txn> txns = new ArrayList<Txn>();
                 //Each op in a multi-op must have the same zxid!
@@ -587,9 +590,7 @@ public class PrepRequestProcessor extends ZooKeeperCriticalThread implements
                         try {
                             pRequest2Txn(op.getType(), zxid, request, subrequest, false);
                         } catch (KeeperException e) {
-                            if (ke == null) {
-                                ke = e;
-                            }
+                            ke = e;
                             request.hdr.setType(OpCode.error);
                             request.txn = new ErrorTxn(e.code().intValue());
                             LOG.info("Got user-level KeeperException when processing "
@@ -616,7 +617,8 @@ public class PrepRequestProcessor extends ZooKeeperCriticalThread implements
                     index++;
                 }
 
-                request.hdr = new TxnHeader(request.sessionId, request.cxid, zxid, zks.getTime(), request.type);
+                request.hdr = new TxnHeader(request.sessionId, request.cxid, zxid,
+                        Time.currentWallTime(), request.type);
                 request.txn = new MultiTxn(txns);
                 
                 break;
@@ -639,6 +641,9 @@ public class PrepRequestProcessor extends ZooKeeperCriticalThread implements
                 zks.sessionTracker.checkSession(request.sessionId,
                         request.getOwner());
                 break;
+            default:
+                LOG.warn("unknown type " + request.type);
+                break;
             }
         } catch (KeeperException e) {
             if (request.hdr != null) {
