@@ -54,4 +54,24 @@ public class LeaderBean extends ZooKeeperServerBean implements LeaderMXBean {
     public long getElectionTimeTaken() {
         return leader.self.getElectionTimeTaken();
     }
+
+    @Override
+    public int getLastProposalSize() {
+        return leader.getProposalStats().getLastProposalSize();
+    }
+
+    @Override
+    public int getMinProposalSize() {
+        return leader.getProposalStats().getMinProposalSize();
+    }
+
+    @Override
+    public int getMaxProposalSize() {
+        return leader.getProposalStats().getMaxProposalSize();
+    }
+
+    @Override
+    public void resetProposalStatistics() {
+        leader.getProposalStats().reset();
+    }
 }
