@@ -140,7 +140,7 @@ public class ObserverZooKeeperServer extends LearnerZooKeeperServer {
 
     @Override
     public synchronized void shutdown() {
-        if (!isRunning()) {
+        if (!canShutdown()) {
             LOG.debug("ZooKeeper server is not running, so not proceeding to shutdown!");
             return;
         }
