@@ -21,7 +21,9 @@ import java.io.File;
 import java.io.IOException;
 
 import javax.management.JMException;
+import javax.security.sasl.SaslException;
 
+import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.apache.zookeeper.jmx.ManagedUtil;
@@ -60,6 +62,7 @@ import org.apache.zookeeper.server.quorum.QuorumPeerConfig.ConfigException;
  * "myid" that contains the server id as an ASCII decimal value.
  *
  */
+@InterfaceAudience.Public
 public class QuorumPeerMain {
     private static final Logger LOG = LoggerFactory.getLogger(QuorumPeerMain.class);
 
@@ -129,18 +132,21 @@ public class QuorumPeerMain {
           ServerCnxnFactory cnxnFactory = ServerCnxnFactory.createFactory();
           cnxnFactory.configure(config.getClientPortAddress(),
                                 config.getMaxClientCnxns());
-  
-          quorumPeer = new QuorumPeer(config.getServers(),
-                                      new File(config.getDataDir()),
-                                      new File(config.getDataLogDir()),
-                                      config.getElectionAlg(),
-                                      config.getServerId(),
-                                      config.getTickTime(),
-                                      config.getInitLimit(),
-                                      config.getSyncLimit(),
-                                      config.getQuorumListenOnAllIPs(),
-                                      cnxnFactory,
-                                      config.getQuorumVerifier());
+
+          quorumPeer = getQuorumPeer();
+
+          quorumPeer.setQuorumPeers(config.getServers());
+          quorumPeer.setTxnFactory(new FileTxnSnapLog(
+                  new File(config.getDataLogDir()),
+                  new File(config.getDataDir())));
+          quorumPeer.setElectionType(config.getElectionAlg());
+          quorumPeer.setMyid(config.getServerId());
+          quorumPeer.setTickTime(config.getTickTime());
+          quorumPeer.setInitLimit(config.getInitLimit());
+          quorumPeer.setSyncLimit(config.getSyncLimit());
+          quorumPeer.setQuorumListenOnAllIPs(config.getQuorumListenOnAllIPs());
+          quorumPeer.setCnxnFactory(cnxnFactory);
+          quorumPeer.setQuorumVerifier(config.getQuorumVerifier());
           quorumPeer.setClientPortAddress(config.getClientPortAddress());
           quorumPeer.setMinSessionTimeout(config.getMinSessionTimeout());
           quorumPeer.setMaxSessionTimeout(config.getMaxSessionTimeout());
@@ -168,4 +174,9 @@ public class QuorumPeerMain {
           LOG.warn("Quorum Peer interrupted", e);
       }
     }
+
+    // @VisibleForTesting
+    protected QuorumPeer getQuorumPeer() throws SaslException {
+        return new QuorumPeer();
+    }
 }
