@@ -34,6 +34,7 @@ import java.net.Socket;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.SynchronousQueue;
 import java.util.concurrent.ThreadPoolExecutor;
@@ -68,6 +69,14 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.mockito.Matchers.any;
+import static org.mockito.Matchers.anyLong;
+import static org.mockito.Matchers.eq;
+import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.timeout;
+import static org.mockito.Mockito.verify;
+
 public class QuorumCnxManagerTest extends ZKTestCase {
     private static final Logger LOG = LoggerFactory.getLogger(QuorumCnxManagerTest.class);
     private int count;
@@ -182,9 +191,9 @@ public class QuorumCnxManagerTest extends ZKTestCase {
 
     /**
      * Peer0 has auth configured, Peer1 has no auth configured.
-     * Peer1 connects to peer0, but is disconnected, because peer1's sid is
+     * Peer0 connects to peer1, but is disconnected, because peer1's sid is
      * higher than peer0.
-     * Peer0 connects to peer1, but is disconnected, because peer1 cannot
+     * Peer1 connects to peer0, but is disconnected, because peer1 cannot
      * handle auth.
      */
     @Test(timeout = 30000)
@@ -218,16 +227,34 @@ public class QuorumCnxManagerTest extends ZKTestCase {
      * No auth learner connects to a server that requires auth, when the server
      * has a higher sid.
      * The connection should fail in both directions.
+     *
+     * peer0 should attempt to connect to peer1, but disconnect as its sid is lower
+     * peer1 should attempt to connect to peer0, peer0 will accept and add an entry to
+     * the senderWorkerMap but peer1 will disconnect because peer1 will start speaking SASL
+     * and peer0 will consider this invalid.
+     *
+     * Due to the unique behavior of peer0 creating an entry
+     * in senderWorkerMap for peer1 and then deleting it we use mockito spies to track
+     * this behavior.
      */
     @Test(timeout = 30000)
     public void testNoAuthLearnerConnectToAuthRequiredServerWithHigherSid()
             throws Exception {
-        QuorumCnxManager peer0 = createAndStartManager(0);
-        QuorumCnxManager peer1 = createAndStartManager(1, "QuorumServer",
-                                                       "QuorumLearner", true, true);
+        ConcurrentHashMap<Long, QuorumCnxManager.SendWorker> senderWorkerMap0 =
+                spy(new ConcurrentHashMap<Long, QuorumCnxManager.SendWorker>());
+        ConcurrentHashMap<Long, QuorumCnxManager.SendWorker> senderWorkerMap1 =
+                spy(new ConcurrentHashMap<Long, QuorumCnxManager.SendWorker>());
+
+        QuorumCnxManager peer0 = createAndStartManager(0, senderWorkerMap0);
+        QuorumCnxManager peer1 = createAndStartManager(1, "QuorumServer", "QuorumLearner",
+                true, true, senderWorkerMap1);
         peer0.connectOne(1);
         peer1.connectOne(0);
-        assertEventuallyNotConnected(peer0, 1);
+
+        verify(senderWorkerMap0, timeout(10000)).put(eq(1L), any(QuorumCnxManager.SendWorker.class));
+        verify(senderWorkerMap0, timeout(10000)).remove(eq(1L), any(QuorumCnxManager.SendWorker.class));
+
+        verify(senderWorkerMap1, never()).put(anyLong(), any(QuorumCnxManager.SendWorker.class));
     }
 
     /**
@@ -679,9 +706,14 @@ public class QuorumCnxManagerTest extends ZKTestCase {
     }
 
     private QuorumCnxManager createAndStartManager(long sid) {
+        return createAndStartManager(sid, new ConcurrentHashMap<Long, QuorumCnxManager.SendWorker>());
+    }
+
+    private QuorumCnxManager createAndStartManager(long sid, ConcurrentHashMap<Long, QuorumCnxManager.SendWorker> senderWorkerMap) {
         QuorumCnxManager peer = new QuorumCnxManager(sid, peers,
                 new NullQuorumAuthServer(), new NullQuorumAuthLearner(), 10000,
-                false, quorumCnxnThreadsSize, false);
+                false, quorumCnxnThreadsSize, false,
+                senderWorkerMap);
         executor.submit(peer.listener);
         InetSocketAddress electionAddr = peer.view.get(sid).electionAddr;
         waitForElectionAddrBinding(electionAddr, 15);
@@ -692,14 +724,24 @@ public class QuorumCnxManagerTest extends ZKTestCase {
                                                    String serverLoginContext,
                                                    String learnerLoginContext,
                                                    boolean serverRequireSasl,
-                                                   boolean learnerRequireSasl)
+                                                   boolean learnerRequireSasl) throws Exception {
+        return createAndStartManager(sid, serverLoginContext, learnerLoginContext, serverRequireSasl, learnerRequireSasl, new ConcurrentHashMap<Long, QuorumCnxManager.SendWorker>());
+
+    }
+
+    private QuorumCnxManager createAndStartManager(long sid,
+                                                   String serverLoginContext,
+                                                   String learnerLoginContext,
+                                                   boolean serverRequireSasl,
+                                                   boolean learnerRequireSasl,
+                                                   ConcurrentHashMap<Long, QuorumCnxManager.SendWorker> senderWorkerMap)
             throws Exception {
         QuorumAuthLearner authClient = new SaslQuorumAuthLearner(learnerRequireSasl,
                 "NOT_USING_KRB_PRINCIPAL", learnerLoginContext);
         QuorumAuthServer authServer = new SaslQuorumAuthServer(serverRequireSasl,
                 serverLoginContext, authzHosts);
         QuorumCnxManager peer = new QuorumCnxManager(sid, peers,
-                authServer, authClient, 10000, false, quorumCnxnThreadsSize, true);
+                authServer, authClient, 10000, false, quorumCnxnThreadsSize, true, senderWorkerMap);
         executor.submit(peer.listener);
         InetSocketAddress electionAddr = peer.view.get(sid).electionAddr;
         waitForElectionAddrBinding(electionAddr, 15);
